
#include "./BMPEncoder.h"

#include <stdio.h>
#include <shlobj.h>


BMPEncoder::BMPEncoder() : _outputDirectory(0) {
}

BMPEncoder::~BMPEncoder() {
    delete _outputDirectory;
}

bool BMPEncoder::onStartEncoding(const char* movieFile) {
    char* movieDirectory = strdup(movieFile);
    int i = strlen(movieDirectory) - 1;
    for (; i >= 0; i--) { if (movieDirectory[i] == '\\' || movieDirectory[i] == '/') break; }
    if (i >= 0) movieDirectory[i] = '\0';

    char outputDirectory[4096];
    if (browseForDirectory(0, outputDirectory, "Please select a directory to output images to", movieDirectory)) {
        _outputDirectory = strdup(outputDirectory);

        _frameCounter = 0;
        delete movieDirectory;
        return true;
    }

    delete movieDirectory;
    return false;
}

void BMPEncoder::clock(const unsigned char* videoFrame, const unsigned char* audioFrame) {
    char outputFile[4096];
    sprintf(outputFile, "%s\\image_%06i.bmp", _outputDirectory, _frameCounter);


    tagBITMAPFILEHEADER fileHeader;
    memset(&fileHeader, 0, sizeof(fileHeader));
    memcpy(&fileHeader.bfType, "BM", 2);
    fileHeader.bfSize = sizeof(BITMAPINFOHEADER);
    fileHeader.bfOffBits = sizeof(BITMAPFILEHEADER) + sizeof(BITMAPINFOHEADER);

    tagBITMAPINFOHEADER infoHeader;
    memset(&infoHeader, 0, sizeof(infoHeader));
    infoHeader.biSize = sizeof(BITMAPINFOHEADER);
    infoHeader.biWidth = 256;
    infoHeader.biHeight = 224;
    infoHeader.biPlanes = 1;
    infoHeader.biBitCount = 24;
    infoHeader.biCompression = BI_RGB;
    infoHeader.biSizeImage = sizeof(RGBTRIPLE) * 256 * 224;
    infoHeader.biXPelsPerMeter = 2400;
    infoHeader.biYPelsPerMeter = 2400;

    FILE* s = fopen(outputFile, "wb");
    fwrite(&fileHeader, sizeof(fileHeader), 1, s);
    fwrite(&infoHeader, sizeof(infoHeader), 1, s);
    for (int y = 223; y >= 0; y--) {
        fwrite(&videoFrame[y * 256 * 3], 256 * 3, 1, s);
    }
    fclose(s);

    _frameCounter++;
}


bool BMPEncoder::browseForDirectory(HWND hwndOwner, LPTSTR lpszDir, LPCTSTR lpszTitle, const char* startDir) {      
	BROWSEINFO bi;
    LPITEMIDLIST pidl;
    IMalloc* pMalloc;

    OleInitialize(NULL);

    if (FAILED(SHGetMalloc(&pMalloc))) return false;

    bi.pidlRoot = NULL;
    bi.hwndOwner = hwndOwner;		    // owner window    
    bi.pszDisplayName = lpszDir;	    // pointer to string that gets folder display name      
	bi.lpszTitle = lpszTitle;		    // pointer to text to display in window
    bi.ulFlags = BIF_RETURNONLYFSDIRS;  // | BIF_USENEWUI ;  // flags - see API docs for BROWSEINFO
    bi.lpfn = browseCallbackProc;	    // callback function - usually not needed
    bi.lParam = (LPARAM) startDir;	    // parameter to callback function

    pidl = SHBrowseForFolder(&bi);

    if(!pidl) return false;

    if(!SHGetPathFromIDList(pidl,lpszDir)) return false;

    if (bi.pidlRoot != NULL) pMalloc->Free( (void *) bi.pidlRoot );
    
    pMalloc->Free(pidl);
    pMalloc->Release();

    return true;
}


// Sets the initial directory displayed by the dialog.  in theory
int __stdcall BMPEncoder::browseCallbackProc(HWND hwnd, UINT uMsg, LPARAM lParam, LPARAM lpData) {
    if (uMsg == BFFM_INITIALIZED) SendMessage(hwnd, BFFM_SETSELECTION, TRUE, lpData);
    return 0;
}
