
#include "./WAVEncoder.h"

#include <shlobj.h>


WAVEncoder::WAVEncoder() : _output(0) {}

WAVEncoder::~WAVEncoder() {
    fclose(_output); // just in case
}

bool WAVEncoder::onStartEncoding(const char* movieFile) {
    // Get output file from user
    char filename[4096] = "";
    OPENFILENAME ofn;
	ZeroMemory(&ofn, sizeof(ofn));
	ofn.lStructSize = sizeof(ofn);
	ofn.hwndOwner = 0;
	ofn.lpstrFilter = "WAV Files (*.wav)\0*.wav\0All Files\0*.*\0";
	ofn.lpstrFile = filename;
	ofn.nMaxFile = 4096;
	ofn.Flags = OFN_EXPLORER | OFN_HIDEREADONLY;
	ofn.lpstrDefExt = "wav";
    ofn.lpstrTitle = "Please select a WAV file to output sound to";

    char* movieDirectory = strdup(movieFile);
    int i = strlen(movieDirectory) - 1;
    for (; i >= 0; i--) { if (movieDirectory[i] == '\\' || movieDirectory[i] == '/') break; }
    if (i >= 0) movieDirectory[i] = '\0';
    ofn.lpstrInitialDir = movieDirectory;;

    bool gotFile = GetSaveFileName(&ofn) == TRUE;
    delete movieDirectory;
    if (!gotFile) return false;

    // Open WAV file for output
    _output = fopen(filename, "wb");
    if (!_output) return false;

    // Write preliminary WAV file header
    {
        fprintf(_output, "RIFF");
        unsigned int fileSize = 0;
        fwrite(&fileSize, 4, 1, _output);
        fprintf(_output, "WAVE");
    }

    // Write format chunk
    {
        unsigned int chunkSize = 16;
        unsigned short format = 1, channels = 1;
        unsigned int sampleRate = 44640, bytesPerSecond = 44640 * 2;
        unsigned short blockAlign = 2, bitsPerSample = 16;
        
        fprintf(_output, "fmt ");        
        fwrite(&chunkSize, sizeof(chunkSize), 1, _output);
        fwrite(&format, sizeof(format), 1, _output);
        fwrite(&channels, sizeof(channels), 1, _output);
        fwrite(&sampleRate, sizeof(sampleRate), 1, _output);
        fwrite(&bytesPerSecond, sizeof(bytesPerSecond), 1, _output);
        fwrite(&blockAlign, sizeof(blockAlign), 1, _output);
        fwrite(&bitsPerSample, sizeof(bitsPerSample), 1, _output);
    }


    // Write preliminary data chunk
    {
        fprintf(_output, "data");
        unsigned int dataSize;
        fwrite(&dataSize, 4, 1, _output);
    }

    _dataChunkSize = 0;

    return true;
}

void WAVEncoder::clock(const unsigned char* videoFrame, const unsigned char* audioFrame) {

    // Write data to output file
    fwrite(audioFrame, 1488, 1, _output);
    
    _dataChunkSize += 1488;
}

bool WAVEncoder::onStopEncoding() {
    // We are done encoding, so now we have to go back and update our file header and data chunk header
    unsigned int fileSize = ftell(_output);
    fileSize -= 4;
    
    fseek(_output, 4, SEEK_SET);
    fwrite(&fileSize, 4, 1, _output);

    fseek(_output, 40, SEEK_SET);
    fwrite(&_dataChunkSize, 4, 1, _output);
    
    fclose(_output);
    return true;
}