// DLL entry point for WAV Encoder plugin

#include <windows.h>
#include "./WAVEncoder.h"

#define DLLExport   __declspec( dllexport )

// Prototypes
extern "C" DLLExport void* OnCreate(HINSTANCE dllInstance, HWND owner);
extern "C" DLLExport void OnDestroy(void* context);
extern "C" DLLExport char* GetName();
extern "C" DLLExport int GetUberNESMovieEncoderPluginVersion();
extern "C" DLLExport void ShowOptions(void* context);
extern "C" DLLExport bool OnStartEncoding(void* context, const char* movieFile);
extern "C" DLLExport bool OnStopEncoding(void * context);
extern "C" DLLExport void OnFrame(void* context, const unsigned char* videoFrame, const unsigned char* audioFrame);



// Create instance of plugin
extern "C" DLLExport void* OnCreate(HINSTANCE instance, HWND owner) {
    return (void*) (new WAVEncoder);
}


// Destroy instance of plugin
extern "C" DLLExport void OnDestroy(void* context) {
    WAVEncoder* encoder = (WAVEncoder*)context;
    delete context;
}

// Get name of plugin
extern "C" DLLExport char* GetName() {
    return ".WAV file encoder";
}

// Get plugin version / validate plugin function
extern "C" DLLExport int GetUberNESMovieEncoderPluginVersion() {
    return 0x10;
}

extern "C" DLLExport void ShowOptions(void* context) {
}

extern "C" bool OnStartEncoding(void* context, const char* movieFile) {
    WAVEncoder* encoder = (WAVEncoder*)context;
    return encoder->onStartEncoding(movieFile);
}

extern "C" bool OnStopEncoding(void* context) {
    WAVEncoder* encoder = (WAVEncoder*)context;
    return encoder->onStopEncoding();
}

// Audio format is 44 KhZ, 16-bit samples.  There are 744 samples per frame (1488 bytes)
extern "C" void OnFrame(void* context, const unsigned char* videoFrame, const unsigned char* audioFrame) {
    WAVEncoder* encoder = (WAVEncoder*)context;
    return encoder->clock(videoFrame, audioFrame);
}