#include "./Settings.h"
#include "./XvidEncoder.h"
#include "../resource.h"

#include <shellapi.h>

namespace XvidSettings {

    namespace Impl {

        struct Context {
            XvidEncoder* plugin;
            HWND owner;
            Context(HWND ownerIn, XvidEncoder* pluginIn) : owner(ownerIn), plugin(pluginIn) {}
        };
        Context* context;


        // sets the dialog controls with the values stored in plugin
        void SyncDialog(HWND hwnd) {
            // Video quality
            SetWindowText(GetDlgItem(hwnd, IDC_EDIT_VIDEO_QUALITY), context->plugin->getOption(OPTION_VIDEO_QUALITY));

            // Video framerate
            {                
                HWND combo = GetDlgItem(hwnd, IDC_COMBO_VIDEO_FRAMERATE);
                SendMessage(combo, CB_ADDSTRING, 0, (LPARAM)"15 Hz");
                SendMessage(combo, CB_ADDSTRING, 0, (LPARAM)"20 Hz");
                SendMessage(combo, CB_ADDSTRING, 0, (LPARAM)"30 Hz");
                SendMessage(combo, CB_ADDSTRING, 0, (LPARAM)"60 Hz");

                char* videoFramerate = context->plugin->getOption(OPTION_VIDEO_FRAMERATE);
                if (!strcmp(videoFramerate, "15")) {
                    SendMessage(combo, CB_SETCURSEL, 0, 0);
                } else if (!strcmp(videoFramerate, "20")) {
                    SendMessage(combo, CB_SETCURSEL, 1, 0);
                } else if (!strcmp(videoFramerate, "30")) {
                    SendMessage(combo, CB_SETCURSEL, 2, 0);
                } else {
                    SendMessage(combo, CB_SETCURSEL, 3, 0);
                }
            }

            // Audio samplerate
            {                
                HWND combo = GetDlgItem(hwnd, IDC_COMBO_AUDIO_SAMPLERATE);
                SendMessage(combo, CB_ADDSTRING, 0, (LPARAM)"11 KHz");
                SendMessage(combo, CB_ADDSTRING, 0, (LPARAM)"22 KHz");
                SendMessage(combo, CB_ADDSTRING, 0, (LPARAM)"44 KHz");

                char* audioSamplerate = context->plugin->getOption(OPTION_AUDIO_SAMPLERATE);
                if (!strcmp(audioSamplerate, "11025")) {
                    SendMessage(combo, CB_SETCURSEL, 0, 0);
                } else if (!strcmp(audioSamplerate, "22050")) {
                    SendMessage(combo, CB_SETCURSEL, 1, 0);
                } else {
                    SendMessage(combo, CB_SETCURSEL, 2, 0);
                }
            }
        }

        void SyncPlugin(HWND hwnd) {
            // Video quality
            char temp[2048];
            GetWindowText(GetDlgItem(hwnd, IDC_EDIT_VIDEO_QUALITY), temp, 2048);
            float videoQuality;
            sscanf(temp, "%f", &videoQuality);
            videoQuality = max(0.0f, min(1.0f, videoQuality));
            sprintf(temp, "%f", videoQuality);
            context->plugin->setOption(OPTION_VIDEO_QUALITY, temp);

            // Video framerate
            {
                HWND combo = GetDlgItem(hwnd, IDC_COMBO_VIDEO_FRAMERATE);
                int curSel = (int)SendMessage(combo, CB_GETCURSEL, 2, 0);
                switch (curSel) {
                    case 0: context->plugin->setOption(OPTION_VIDEO_FRAMERATE, "15"); break;
                    case 1: context->plugin->setOption(OPTION_VIDEO_FRAMERATE, "20"); break;
                    case 2: context->plugin->setOption(OPTION_VIDEO_FRAMERATE, "30"); break;
                    case 3: context->plugin->setOption(OPTION_VIDEO_FRAMERATE, "60"); break;
                }
            }

            // Audio samplerate
            {
                HWND combo = GetDlgItem(hwnd, IDC_COMBO_AUDIO_SAMPLERATE);
                int curSel = (int)SendMessage(combo, CB_GETCURSEL, 2, 0);
                switch (curSel) {
                    case 0: context->plugin->setOption(OPTION_AUDIO_SAMPLERATE, "11025"); break;
                    case 1: context->plugin->setOption(OPTION_AUDIO_SAMPLERATE, "22050"); break;
                    case 2: context->plugin->setOption(OPTION_AUDIO_SAMPLERATE, "44100"); break;
                }
            }
        }


        BOOL CALLBACK DlgProc(HWND hwnd, UINT Message, WPARAM wParam, LPARAM lParam) {
	        switch (Message) {
                case WM_INITDIALOG:
                    SyncDialog(hwnd);
		            break;

	            case WM_COMMAND:
		            switch(LOWORD(wParam)) {
                        case IDC_RESTORE_DEFAULTS:
                            context->plugin->clearOptions();
                            context->plugin->setDefaultOptions();
                            SyncDialog(hwnd);
                            break;

                        case IDOK:
                            SyncPlugin(hwnd);
                            EndDialog(hwnd, IDOK);
                            break;

                        case IDC_URL_UBERNES: ShellExecute(0, "open", "http://www.ubernes.com", 0, 0, SW_SHOW); break;
                        case IDC_URL_REVEL: ShellExecute(0, "open", "http://revel.sourceforge.net/", 0, 0, SW_SHOW); break;
                        case IDC_URL_XVID: ShellExecute(0, "open", "http://www.free-codecs.com/FFDShow_download.htm", 0, 0, SW_SHOW); break;
		            }
                    break;

	            default:
		            return FALSE;
            }
	        return TRUE;
        }


    }

    void Show(HINSTANCE instance, HWND owner, XvidEncoder* plugin) {
        Impl::context = new XvidSettings::Impl::Context(owner, plugin);

        DialogBox(instance, MAKEINTRESOURCE(IDD_SETTINGS), owner, Impl::DlgProc);

        delete Impl::context;
    }

}