// DLL entry point for Xvid encoder

#include <stdio.h>
#include <windows.h>

#include "./XvidEncoder.h"

#define DLLExport   __declspec( dllexport )

// Prototypes
extern "C" DLLExport void* OnCreate(HINSTANCE dllInstance, HWND owner);
extern "C" DLLExport void OnDestroy(void* context);
extern "C" DLLExport char* GetName();
extern "C" DLLExport int GetUberNESMovieEncoderPluginVersion();
extern "C" DLLExport void ShowOptions(void* context);
extern "C" DLLExport bool OnStartEncoding(void* context, const char* movieFile);
extern "C" DLLExport bool OnStopEncoding(void * context);

// Video format is 24-bit color, RGB format.  Image is 256x224, so video buffer size is 256 * 224 * 3 = 172032 bytes
// Audio format is 44 KhZ, 16-bit samples.  There are 744 samples per frame (1488 bytes)
extern "C" DLLExport void OnFrame(void* context, const unsigned char* videoFrame, const unsigned char* audioFrame);




// Create instance of plugin
extern "C" DLLExport void* OnCreate(HINSTANCE dllInstance, HWND owner) {
    return (void*) (new XvidEncoder(dllInstance, owner));
}

extern "C" DLLExport void OnDestroy(void* context) {
    XvidEncoder* encoder = (XvidEncoder*)context;
    delete encoder;
}

// Get name of plugin
extern "C" DLLExport char* GetName() {
    return "Xvid Encoder";
}

// Get plugin version / validate plugin function
extern "C" DLLExport int GetUberNESMovieEncoderPluginVersion() {
    return 0x10;
}

extern "C" DLLExport void ShowOptions(void* context) {
    XvidEncoder* encoder = (XvidEncoder*)context;
    encoder->showOptions();
}

extern "C" bool OnStartEncoding(void* context, const char* movieFile) {
    XvidEncoder* encoder = (XvidEncoder*)context;
    return encoder->onStartEncoding(movieFile);
}

extern "C" bool OnStopEncoding(void* context) {
    XvidEncoder* encoder = (XvidEncoder*)context;
    return encoder->onStopEncoding();
}

// Video format is 24-bit color, RGB format.  Image is 256x224, so video buffer size is 256 * 224 * 3 = 172032 bytes
// Audio format is 44 KhZ, 16-bit samples.  There are 744 samples per frame (1488 bytes)
extern "C" void OnFrame(void* context, const unsigned char* videoFrame, const unsigned char* audioFrame) {
    XvidEncoder* encoder = (XvidEncoder*)context;
    return encoder->onFrame(videoFrame, audioFrame);
}